import pystray
from pystray import MenuItem as Item
from PIL import Image
from time import sleep
from pathlib import Path
from ktu_mon.utils.dbops import fetch_config
from ktu_mon.utils import scraping
from ktu_mon.gui import run_gui, GUI_DIR
from ktu_mon.utils.constants import APP_NAME, PROC_ID_SYSTRAY, PROC_ID_TIMER
import concurrent.futures
import multiprocessing
import os

ICON_FILE = str(Path(GUI_DIR, "static", "icons", "ktu-mon-32x32.ico"))


def systray():
    image = Image.open(ICON_FILE)
    appmenu = pystray.Menu(
        Item("Open", on_clicked, default=True, visible=False),
        Item("Check Now!", on_clicked),
        Item("Open Interface", on_clicked),
        pystray.Menu.SEPARATOR,
        Item("Shutdown", lambda: stop(icon)),
    )
    icon = pystray.Icon(name=APP_NAME, icon=image, title=APP_NAME, menu=appmenu)

    def routine_check():
        with concurrent.futures.ThreadPoolExecutor() as executor:
            while icon.visible:
                timer_sec = 60 * int(fetch_config("time_interval")[0][1])
                executor.submit(timer)
                while timer_sec != 0:
                    if icon.visible:
                        if fetch_config("notification_ind")[0][1] == "1":
                            icon.notify("KTU Mon has new notifications for you!")
                            print("KTU Mon has new notifications for you!")
                        sleep(60)
                        timer_sec -= 60
                    else:
                        executor.shutdown(wait=False, cancel_futures=True)
                        break

    def stop(icon):
        icon.visible = False
        print("Shutting down...")
        icon.stop()
        os._exit(0)

    def setup(icon):
        icon.visible = True
        routine_check()

    icon.run(setup=setup)


def cre_gui_proc():
    return multiprocessing.Process(target=run_gui)


def on_clicked(icon, item):
    gui_process = cre_gui_proc()
    if str(item) == "Open":
        print("Web interface is opened!")
        gui_process.start()
        try:
            gui_process.join()
        except KeyboardInterrupt as e:
            print(e)
        print("GUI Server Stopped!")
    elif str(item) == "Check Now!":
        print("Checking!")
        scraping(PROC_ID_SYSTRAY)
    elif str(item) == "Open Interface":
        print("Web interface is opened!")
        gui_process.start()
        try:
            gui_process.join()
        except KeyboardInterrupt as e:
            print(e)
        print("GUI Server Stopped!")
    else:
        print("Not implemented!")


def timer():
    sleep(60 * int(fetch_config("time_interval")[0][1]))
    print("Checking!")
    scraping(PROC_ID_TIMER)



from datetime import datetime
from ktu_mon.utils.scraper import scrape_it
from ktu_mon.utils.dbops import (
    db_init,
    insert_log,
    insert_log_date,
    config_mod,
    fetch_config,
    fetch_comp,
    fetch_log_list,
    fetch_log,
    delete_log,
)
from ktu_mon.utils.constants import PROC_ID_INITIALIZER

scraper_active_ind = 0


def get_time() -> str:
    return datetime.now().strftime("%Y-%m-%dT%H:%M:%S")


def init_db():
    try:
        db_init(get_time)
        scraping(PROC_ID_INITIALIZER)
    except Exception as e:
        print(e)


def scraping(proc_id) -> None:
    global scraper_active_ind
    if scraper_active_ind == 0:
        scraper_active_ind = 1
        data = scrape_it(get_time)
        time = get_time()
        for notif in data[0]:
            insert_log(notif[0], notif[1], proc_id, time, "temp")
        config_mod("last_checked", time, proc_id, get_time)
        added, removed, _ = log_insight(None, fetch_config("last_logged")[0][1], "temp")
        if added + removed > 0:
            for notif in data[0]:
                insert_log(notif[0], notif[1], proc_id, time, None)
            insert_log_date(time, proc_id, time)
            config_mod("last_logged", time, proc_id, get_time)
            config_mod("notification_ind", "1", proc_id, get_time)
            delete_log(time, "temp")
        else:
            delete_log(time, "temp")
        scraper_active_ind = 0
    else:
        print("Scraper in use!")


def log_insight(left_ts, right_ts, mode) -> tuple:
    if mode == "temp":
        data = fetch_comp(None, right_ts, "temp")
    else:
        data = fetch_comp(left_ts, right_ts, None)
    added_log_count = 0
    removed_log_count = 0
    notifications = list()
    for notification in data:
        if notification[2] is None:
            added_log_count += 1
            notifications.append([notification[0], notification[1], 1])
        elif notification[1] is None:
            removed_log_count += 1
            notifications.append([notification[0], notification[2], 2])
        else:
            notifications.append([notification[0], notification[1], 0])
    return (added_log_count, removed_log_count, notifications)


def log_deleter(date) -> None:
    delete_log(date, None)
    if fetch_config("last_logged")[0][1] == date:
        try:
            config_mod(
                "last_logged",
                fetch_log_list(0)[0][0],
                PROC_ID_INITIALIZER,
                get_time,
            )
        except IndexError:
            pass


def data_nodes() -> list:
    additions = list()
    removals = list()
    dates = list()
    length = len(fetch_log_list(0))
    added = int()
    removed = int()
    logs = list()
    total_logs = list()
    for i in range(length):  # type: ignore
        try:
            added, removed, logs = log_insight(fetch_log_list(0)[i][0], fetch_log_list(0)[i + 1][0], None)
        except IndexError:
            pass
        try:
            additions.append(added)
            removals.append(removed)
            dates.append(
                datetime.strptime(fetch_log_list(0)[i][0], "%Y-%m-%dT%H:%M:%S").strftime("%d %b %Y %I:%M %p")
            )
        except UnboundLocalError:
            pass
    try:
        dates.pop()
        additions.pop()
        removals.pop()
    except IndexError:
        pass

    for i in fetch_log(fetch_config("last_logged")[0][1], mode=None):
        total_logs.append(i[0])

    dateset = [*set(total_logs)]
    dateset.sort(key=lambda date: datetime.strptime(date, "%Y-%m-%dT%H:%M:%S"))

    full_log_insight = list()
    log_dates = list()
    log_count = list()
    for date in dateset:
        count = total_logs.count(date)
        log_dates.append(datetime.strptime(date, "%Y-%m-%dT%H:%M:%S").strftime("%d %b %Y"))
        log_count.append(count)

    full_log_insight.append(log_dates)
    full_log_insight.append(log_count)
    dates.reverse()
    additions.reverse()
    removals.reverse()
    return [dates, additions, removals, full_log_insight]



from requests import Response, get
from bs4 import BeautifulSoup, Tag
from ktu_mon.utils.constants import PAGE_URL, TABLE_CLASS, DATE_CLASS
from datetime import datetime


def scrape_it(time_func) -> tuple[list[tuple], str]:
    page: Response = get(PAGE_URL)
    soup: BeautifulSoup = BeautifulSoup(page.text, "html.parser")
    table: Tag = soup.find("table", class_=TABLE_CLASS)  # type: ignore
    notification_dates: list[str] = list()
    notification_titles: list[str] = list()
    timestamp: str = time_func()
    for notification in table.find_all("tr"):
        for label in notification.find_all("label", class_=DATE_CLASS):
            notification_dates.append(
                datetime.strptime(str(label.text), "%a %b %d %H:%M:%S IST %Y").strftime("%Y-%m-%dT%H:%M:%S")
            )
        for td in notification.find_all("td"):
            for li in td.find_all("li"):
                for b in li.find_all("b", limit=1):
                    notification_titles.append(b.text)

    notification_tuple: list[tuple] = list()
    for i in range(len(notification_titles)):
        notification_tuple.append((notification_dates[i], notification_titles[i]))

    return (notification_tuple, timestamp)



import sqlite3
from sqlite3 import Connection, Cursor
from typing import Callable
from pathlib import Path
from ktu_mon.utils.constants import DB_NAME, PROC_ID_INITIALIZER


def db_init(time_func: Callable[[], str]) -> None:
    if Path(f"{DB_NAME}.db").is_file():
        print("Local DB exists! Skipping Initialisation.")
        raise Exception("DB EXISTS!")
    else:
        print("Local DB does not exist! Creating!")
        conn: Connection = sqlite3.connect(f"{DB_NAME}.db")
        cur: Cursor = conn.cursor()
        with conn:
            cur.execute(
                """CREATE TABLE log_temp (
                    notif_date text not null,
                    notification text not null,
                    cre_proc integer not null,
                    cre_proc_ts text not null,
                    upd_proc integer,
                    upd_proc_ts text,
                    trashed integer not null
                )"""
            )
            cur.execute(
                """CREATE TABLE logs (
                    notif_date text not null,
                    notification text not null,
                    cre_proc integer not null,
                    cre_proc_ts text not null,
                    upd_proc integer,
                    upd_proc_ts text,
                    trashed integer not null
                )"""
            )
            cur.execute(
                """CREATE TABLE log_dates (
                    notif_date text not null,
                    cre_proc integer not null,
                    cre_proc_ts text primary key,
                    upd_proc integer,
                    upd_proc_ts text,
                    trashed integer not null
                )"""
            )
            cur.execute(
                """CREATE TABLE config (
                    parameter text primary key,
                    value text,
                    cre_proc integer not null,
                    cre_proc_ts text not null,
                    upd_proc integer,
                    upd_proc_ts text
                )"""
            )
            cur.execute(
                """INSERT INTO config (
                    parameter,
                    value,
                    cre_proc,
                    cre_proc_ts,
                    upd_proc,
                    upd_proc_ts
                ) VALUES (
                    :parameter,
                    :value,
                    :cre_proc,
                    :cre_proc_ts,
                    :upd_proc,
                    :upd_proc_ts
                )""",
                {
                    "parameter": "time_interval",
                    "value": 30,
                    "cre_proc": PROC_ID_INITIALIZER,
                    "cre_proc_ts": time_func(),
                    "upd_proc": None,
                    "upd_proc_ts": None,
                },
            )
            cur.execute(
                """INSERT INTO config (
                    parameter,
                    value,
                    cre_proc,
                    cre_proc_ts,
                    upd_proc,
                    upd_proc_ts
                ) VALUES (
                    :parameter,
                    :value,
                    :cre_proc,
                    :cre_proc_ts,
                    :upd_proc,
                    :upd_proc_ts
                )""",
                {
                    "parameter": "last_checked",
                    "value": None,
                    "cre_proc": PROC_ID_INITIALIZER,
                    "cre_proc_ts": time_func(),
                    "upd_proc": None,
                    "upd_proc_ts": None,
                },
            )
            cur.execute(
                """INSERT INTO config (
                    parameter,
                    value,
                    cre_proc,
                    cre_proc_ts,
                    upd_proc,
                    upd_proc_ts
                ) VALUES (
                    :parameter,
                    :value,
                    :cre_proc,
                    :cre_proc_ts,
                    :upd_proc,
                    :upd_proc_ts
                )""",
                {
                    "parameter": "last_logged",
                    "value": None,
                    "cre_proc": PROC_ID_INITIALIZER,
                    "cre_proc_ts": time_func(),
                    "upd_proc": None,
                    "upd_proc_ts": None,
                },
            )
            cur.execute(
                """INSERT INTO config (
                    parameter,
                    value,
                    cre_proc,
                    cre_proc_ts,
                    upd_proc,
                    upd_proc_ts
                ) VALUES (
                    :parameter,
                    :value,
                    :cre_proc,
                    :cre_proc_ts,
                    :upd_proc,
                    :upd_proc_ts
                )""",
                {
                    "parameter": "notification_ind",
                    "value": "0",
                    "cre_proc": PROC_ID_INITIALIZER,
                    "cre_proc_ts": time_func(),
                    "upd_proc": None,
                    "upd_proc_ts": None,
                },
            )


def config_mod(parameter: str, value: str, proc_id: int, time_func: Callable[[], str]) -> None:
    conn: Connection = sqlite3.connect(f"{DB_NAME}.db")
    cur: Cursor = conn.cursor()
    with conn:
        cur.execute(
            """UPDATE config SET
                value = :value,
                upd_proc = :upd_proc,
                upd_proc_ts = :upd_proc_ts
                WHERE parameter = :parameter""",
            {
                "value": value,
                "parameter": parameter,
                "upd_proc": proc_id,
                "upd_proc_ts": time_func(),
            },
        )
    print(f"Config {parameter} set to {value}")


def insert_log(date: str, notification: str, proc_id: int, proc_ts: str, mode: str | None) -> None:
    conn: Connection = sqlite3.connect(f"{DB_NAME}.db")
    cur: Cursor = conn.cursor()
    with conn:
        if mode == "temp":
            cur.execute(
                """INSERT INTO log_temp (
                        notif_date,
                        notification,
                        cre_proc,
                        cre_proc_ts,
                        upd_proc,
                        upd_proc_ts,
                        trashed
                    ) VALUES (
                        :notif_date,
                        :notification,
                        :cre_proc,
                        :cre_proc_ts,
                        :upd_proc,
                        :upd_proc_ts,
                        :trashed
                    )""",
                {
                    "notif_date": date,
                    "notification": notification,
                    "cre_proc": proc_id,
                    "cre_proc_ts": proc_ts,
                    "upd_proc": None,
                    "upd_proc_ts": None,
                    "trashed": 0,
                },
            )
        else:
            cur.execute(
                """INSERT INTO logs (
                        notif_date,
                        notification,
                        cre_proc,
                        cre_proc_ts,
                        upd_proc,
                        upd_proc_ts,
                        trashed
                    ) VALUES (
                        :notif_date,
                        :notification,
                        :cre_proc,
                        :cre_proc_ts,
                        :upd_proc,
                        :upd_proc_ts,
                        :trashed
                    )""",
                {
                    "notif_date": date,
                    "notification": notification,
                    "cre_proc": proc_id,
                    "cre_proc_ts": proc_ts,
                    "upd_proc": None,
                    "upd_proc_ts": None,
                    "trashed": 0,
                },
            )


def insert_log_date(date, proc_id, proc_ts) -> None:
    conn: Connection = sqlite3.connect(f"{DB_NAME}.db")
    cur: Cursor = conn.cursor()
    with conn:
        cur.execute(
            """INSERT INTO log_dates (
                notif_date,
                cre_proc,
                cre_proc_ts,
                upd_proc,
                upd_proc_ts,
                trashed
            ) VALUES (
                :notif_date,
                :cre_proc,
                :cre_proc_ts,
                :upd_proc,
                :upd_proc_ts,
                :trashed
            )""",
            {
                "notif_date": date,
                "cre_proc": proc_id,
                "cre_proc_ts": proc_ts,
                "upd_proc": None,
                "upd_proc_ts": None,
                "trashed": 0,
            },
        )
        print(f"Inserted log dated {date}")


def trash_log(date, proc_id, time_func) -> None:
    conn: Connection = sqlite3.connect(f"{DB_NAME}.db")
    cur: Cursor = conn.cursor()
    with conn:
        cur.execute(
            """UPDATE logs SET
                trashed = 1,
                upd_proc = :upd_proc
                upd_proc_ts = :upd_proc_ts
                WHERE cre_proc_ts = :cre_proc_ts""",
            {
                "upd_proc": proc_id,
                "upd_proc_ts": time_func(),
                "cre_proc_ts": date,
            },
        )
        cur.execute(
            """UPDATE log_dates SET
                trashed = 1,
                upd_proc = :upd_proc
                upd_proc_ts = :upd_proc_ts
                WHERE cre_proc_ts = :cre_proc_ts""",
            {
                "upd_proc": proc_id,
                "upd_proc_ts": time_func(),
                "cre_proc_ts": date,
            },
        )
        print(f"Log dated {date} is now in trash")


def restore_log(date, proc_id, time_func) -> None:
    conn: Connection = sqlite3.connect(f"{DB_NAME}.db")
    cur: Cursor = conn.cursor()
    with conn:
        cur.execute(
            """UPDATE logs SET
                trashed = 0,
                upd_proc = :upd_proc
                upd_proc_ts = :upd_proc_ts
                WHERE cre_proc_ts = :cre_proc_ts""",
            {
                "upd_proc": proc_id,
                "upd_proc_ts": time_func(),
                "cre_proc_ts": date,
            },
        )
        cur.execute(
            """UPDATE log_dates SET
                trashed = 0,
                upd_proc = :upd_proc
                upd_proc_ts = :upd_proc_ts
                WHERE cre_proc_ts = :cre_proc_ts""",
            {
                "upd_proc": proc_id,
                "upd_proc_ts": time_func(),
                "cre_proc_ts": date,
            },
        )
        print(f"Log dated {date} is now restored from trash")


def delete_log(date, mode) -> None:
    conn: Connection = sqlite3.connect(f"{DB_NAME}.db")
    cur: Cursor = conn.cursor()
    with conn:
        if mode == "temp":
            cur.execute(
                """DELETE from log_temp WHERE cre_proc_ts = :cre_proc_ts""",
                {
                    "cre_proc_ts": date,
                },
            )
            print(f"Temporary log cache dated {date} cleared")
        else:
            cur.execute(
                """DELETE from logs WHERE cre_proc_ts = :cre_proc_ts""",
                {
                    "cre_proc_ts": date,
                },
            )
            cur.execute(
                """DELETE from log_dates WHERE cre_proc_ts = :cre_proc_ts""",
                {
                    "cre_proc_ts": date,
                },
            )
            print(f"Log dated {date} is permanently deleted")


def fetch_log_list(trashed) -> list:
    conn: Connection = sqlite3.connect(f"{DB_NAME}.db")
    cur: Cursor = conn.cursor()
    with conn:
        cur.execute(
            """SELECT * FROM log_dates where trashed = :trashed ORDER BY notif_date DESC""",
            {"trashed": trashed},
        )
        return cur.fetchall()


def fetch_log(cre_proc_ts, mode) -> list:
    conn: Connection = sqlite3.connect(f"{DB_NAME}.db")
    cur: Cursor = conn.cursor()
    with conn:
        if mode == "temp":
            cur.execute(
                """SELECT * FROM log_temp where cre_proc_ts = :cre_proc_ts""",
                {"cre_proc_ts": cre_proc_ts},
            )
        else:
            cur.execute(
                """SELECT * FROM logs where cre_proc_ts = :cre_proc_ts""",
                {"cre_proc_ts": cre_proc_ts},
            )
        return cur.fetchall()


def fetch_config(key) -> list:
    conn: Connection = sqlite3.connect(f"{DB_NAME}.db")
    cur: Cursor = conn.cursor()
    with conn:
        cur.execute(
            """SELECT * FROM config where parameter = :parameter""",
            {"parameter": key},
        )
        return cur.fetchall()


def fetch_comp(left_ts, right_ts, mode) -> list:
    conn: Connection = sqlite3.connect(f"{DB_NAME}.db")
    cur: Cursor = conn.cursor()
    with conn:
        if mode == "temp":
            cur.execute(
                """SELECT _left.notif_date as notification_ts,
                _left.notification as _left,
                _right.notification as _right
                FROM (SELECT * FROM log_temp) as _left
                LEFT JOIN (SELECT * FROM logs WHERE cre_proc_ts=:right_ts) as _right
                ON _left.notification=_right.notification AND _left.notif_date=_right.notif_date
                UNION ALL
                SELECT _right.notif_date as notification_ts,
                _left.notification as _left,
                _right.notification as _right
                FROM (SELECT * FROM logs WHERE cre_proc_ts=:right_ts) as _right
                LEFT JOIN (SELECT * FROM log_temp) as _left
                ON _left.notification=_right.notification AND _left.notif_date=_right.notif_date
                WHERE _left.cre_proc_ts IS NULL
                ORDER BY notification_ts DESC
                """,
                {"right_ts": right_ts},
            )
        else:
            cur.execute(
                """SELECT _left.notif_date as notification_ts,
                _left.notification as _left,
                _right.notification as _right
                FROM (SELECT * FROM logs WHERE cre_proc_ts=:left_ts) as _left
                LEFT JOIN (SELECT * FROM logs WHERE cre_proc_ts=:right_ts) as _right
                ON _left.notification=_right.notification AND _left.notif_date=_right.notif_date
                UNION ALL
                SELECT _right.notif_date as notification_ts,
                _left.notification as _left,
                _right.notification as _right
                FROM (SELECT * FROM logs WHERE cre_proc_ts=:right_ts) as _right
                LEFT JOIN (SELECT * FROM logs WHERE cre_proc_ts=:left_ts) as _left
                ON _left.notification=_right.notification AND _left.notif_date=_right.notif_date
                WHERE _left.cre_proc_ts IS NULL
                ORDER BY notification_ts DESC
                """,
                {"left_ts": left_ts, "right_ts": right_ts},
            )
        return cur.fetchall()



import uvicorn
from typing import List
from fastapi import FastAPI, Request, WebSocket, WebSocketDisconnect, Form
from fastapi.responses import HTMLResponse
from fastapi.staticfiles import StaticFiles
from fastapi.templating import Jinja2Templates
from pathlib import Path
from ktu_mon.utils import (
    log_insight,
    fetch_config,
    scraping,
    get_time,
    log_deleter,
    data_nodes,
    init_db,
)
from ktu_mon.utils.dbops import fetch_log_list, config_mod
from ktu_mon.utils.constants import PROC_ID_WEBGUI
from datetime import datetime
from time import sleep
import concurrent.futures
import webbrowser
import os
import signal
import threading

GUI_DIR = Path(__file__).resolve().parent

gui_app = FastAPI()
gui_app.mount(
    "/static",
    StaticFiles(directory=str(Path(GUI_DIR, "static"))),
    name="static",
)
templates = Jinja2Templates(directory=str(Path(GUI_DIR, "templates")))
templates.env.filters["formatdate"] = lambda value: datetime.strptime(value, "%Y-%m-%d").strftime("%d %b %Y")
templates.env.filters["formattime"] = lambda value: datetime.strptime(value, "%H:%M:%S").strftime("%I:%M %p")

init_db()


def ws_check_timer():
    for i in reversed(range(3)):
        sleep(1)
        print(f"Countdown: {i}")
    if len(manager.active_connections) == 0:
        pid = os.getpid()
        os.kill(pid, signal.CTRL_C_EVENT)
    else:
        pass


def cre_ws_timer_thread():
    return threading.Thread(target=ws_check_timer)


class ConnectionManager:
    def __init__(self):
        self.active_connections: List[WebSocket] = []

    async def connect(self, websocket: WebSocket):
        await websocket.accept()
        self.active_connections.append(websocket)

    def disconnect(self, websocket: WebSocket):
        thread = cre_ws_timer_thread()
        thread.start()
        self.active_connections.remove(websocket)

    async def send_personal_message(self, message: str, websocket: WebSocket):
        await websocket.send_text(message)

    async def broadcast(self, message: str):
        for connection in self.active_connections:
            await connection.send_text(message)


manager = ConnectionManager()


def data_init(vars: dict):
    vars["time_interval"] = [
        str(int(int(fetch_config("time_interval")[0][1]) / 60)),
        str(int(int(fetch_config("time_interval")[0][1]) % 60)),
    ]
    vars["log_list"] = fetch_log_list(0)
    vars["log_count"] = len(vars["log_list"])
    vars["last_checked"] = fetch_config("last_checked")[0][1]
    vars["last_logged"] = fetch_config("last_logged")[0][1]
    try:
        vars["second_last_log"] = fetch_log_list(0)[1][0]
    except Exception:
        vars["second_last_log"] = fetch_config("last_logged")[0][1]
    added, removed, logs = log_insight(vars["last_logged"], vars["second_last_log"], None)
    vars["fetched_logs"] = logs
    vars["removed_notification"] = removed
    vars["added_notification"] = added
    vars["total_notification_count"] = removed + added
    vars["log1"] = vars["last_logged"]
    vars["log2"] = vars["second_last_log"]


def data_mod(vars: dict):
    if (
        len([item for item in fetch_log_list(0) if item[0] == vars["log1"]]) == 0
        or len([item for item in fetch_log_list(0) if item[0] == vars["log2"]]) == 0
    ):
        data_init(vars)
    else:
        vars["time_interval"] = [
            str(int(int(fetch_config("time_interval")[0][1]) / 60)),
            str(int(int(fetch_config("time_interval")[0][1]) % 60)),
        ]
        vars["log_list"] = fetch_log_list(0)
        vars["log_count"] = len(vars["log_list"])
        vars["last_checked"] = fetch_config("last_checked")[0][1]
        vars["last_logged"] = fetch_config("last_logged")[0][1]
        try:
            vars["second_last_log"] = fetch_log_list(0)[1][0]
        except Exception:
            vars["second_last_log"] = fetch_config("last_logged")[0][1]
        added, removed, logs = log_insight(vars["log1"], vars["log2"], None)
        vars["fetched_logs"] = logs
        vars["removed_notification"] = removed
        vars["added_notification"] = added
        vars["total_notification_count"] = removed + added


vars: dict = dict()
data_init(vars)


@gui_app.websocket("/ws/{client_id}")
async def websocket_endpoint(websocket: WebSocket, client_id: int):
    await manager.connect(websocket)
    print("Connected")
    print(f"Active connections: {len(manager.active_connections)}")
    try:
        while True:
            data = await websocket.receive_text()
            await manager.broadcast(f"Received from {client_id}: {data}")

    except WebSocketDisconnect:
        manager.disconnect(websocket)
        await manager.broadcast(f"{client_id} Disconnected")
        print("Disconnected")
        print(f"Active connections: {len(manager.active_connections)}")


@gui_app.get("/", response_class=HTMLResponse)
async def home(request: Request):
    data_mod(vars)
    return templates.TemplateResponse(
        "home.html",
        {"request": request, "vars": vars, "datetime": datetime},
    )


@gui_app.get("/notifications", response_class=HTMLResponse)
async def notifications(request: Request):
    data_mod(vars)
    return templates.TemplateResponse(
        "notifications.html",
        {
            "request": request,
            "vars": vars,
        },
    )


@gui_app.post("/logselect", response_class=HTMLResponse)
async def log_check(request: Request, log1=Form(...), log2=Form(...)):
    vars["log1"] = log1
    vars["log2"] = log2
    data_mod(vars)
    return templates.TemplateResponse(
        "notifications.html",
        {
            "request": request,
            "vars": vars,
        },
    )


@gui_app.get("/added-notifications", response_class=HTMLResponse)
async def added_notifications(request: Request):
    data_mod(vars)
    return templates.TemplateResponse(
        "added_notifications.html",
        {
            "request": request,
            "vars": vars,
        },
    )


@gui_app.get("/removed-notifications", response_class=HTMLResponse)
async def removed_notifications(request: Request):
    data_mod(vars)
    return templates.TemplateResponse(
        "removed_notifications.html",
        {
            "request": request,
            "vars": vars,
        },
    )


@gui_app.get("/logs", response_class=HTMLResponse)
async def logs(request: Request):
    data_mod(vars)
    return templates.TemplateResponse(
        "logs.html",
        {
            "request": request,
            "vars": vars,
        },
    )


@gui_app.get("/insights", response_class=HTMLResponse)
async def insight(request: Request):
    data = data_nodes()
    return templates.TemplateResponse(
        "insights.html",
        {"request": request, "vars": vars, "data": data},
    )


@gui_app.get("/trash-log/{log_id}", response_class=HTMLResponse)
async def log_deletion(request: Request, log_id: str):
    log_deleter(log_id)  # TODO : replace with trash
    data_mod(vars)
    return templates.TemplateResponse(
        "logs.html",
        {
            "request": request,
            "vars": vars,
        },
    )


@gui_app.post("/refresher", response_class=HTMLResponse)
async def refresher(request: Request):
    scraping(PROC_ID_WEBGUI)
    data_init(vars)
    return templates.TemplateResponse(
        "notifications.html",
        {
            "request": request,
            "vars": vars,
        },
    )


@gui_app.post("/timerset", response_class=HTMLResponse)
async def timerset(request: Request, hh: str = Form(...), mm: str = Form(...)):
    time = int(hh) * 60 + int(mm)
    config_mod("time_interval", str(time), PROC_ID_WEBGUI, get_time)
    data_mod(vars)
    return templates.TemplateResponse(
        "notifications.html",
        {
            "request": request,
            "vars": vars,
        },
    )


def run_gui():
    with concurrent.futures.ThreadPoolExecutor() as executor:
        if fetch_config("notification_ind")[0][1] == "1":
            config_mod("notification_ind", "0", PROC_ID_WEBGUI, get_time)
        executor.submit(open_browser)
        uvicorn.run("ktu_mon.gui:gui_app")


def open_browser():
    sleep(1)
    webbrowser.open("http://127.0.0.1:8000/notifications")



APP_NAME = "KTU Mon"
VERSION = "1.0.0"
DB_NAME = "ktu_mon"
SYSTRAY_ICON_FILENAME = "ktu-mon-32x32.ico"
PAGE_URL = "https://ktu.edu.in/eu/core/announcements.htm"
TABLE_CLASS = "ktu-news"
DATE_CLASS = "news-date"
PROC_ID_INITIALIZER = 1
PROC_ID_SYSTRAY = 2
PROC_ID_WEBGUI = 3
PROC_ID_TIMER = 4
